@@ -248,7 +248,7 @@ filter_mask <- function(.data) {
   if (query_on_dataset(.data)) {
     comp_func <- function(operator) {
       force(operator)
-      function(e1, e2) make_expression(operator, e1, e2)
+      function(e1, e2) build_dataset_expression(operator, e1, e2)
     }
     var_binder <- function(x) Expression$field_ref(x)
   } else {
@@ -264,7 +264,10 @@ filter_mask <- function(.data) {
   env_bind(f_env, !!!lapply(func_names, comp_func))
   # Then add the column references
   # Renaming is handled automatically by the named list
-  env_bind(f_env, !!!lapply(.data$selected_columns, var_binder))
+  data_pronoun <- lapply(.data$selected_columns, var_binder)
+  env_bind(f_env, !!!data_pronoun)
+  # Then bind the data pronoun
+  env_bind(f_env, .data = data_pronoun)
   new_data_mask(f_env)
 }
 
@@ -285,15 +288,18 @@ collect.arrow_dplyr_query <- function(x, as_data_frame = TRUE, ...) {
   # Pull only the selected rows and cols into R
   if (query_on_dataset(x)) {
     # See dataset.R for Dataset and Scanner(Builder) classes
-    df <- Scanner$create(x)$ToTable()
+    tab <- Scanner$create(x)$ToTable()
   } else {
     # This is a Table/RecordBatch. See record-batch.R for the [ method
-    df <- x$.data[x$filtered_rows, x$selected_columns, keep_na = FALSE]
+    tab <- x$.data[x$filtered_rows, x$selected_columns, keep_na = FALSE]
   }
   if (as_data_frame) {
-    df <- as.data.frame(df)
+    df <- as.data.frame(tab)
+    tab$invalidate()
+    restore_dplyr_features(df, x)
+  } else {
+    restore_dplyr_features(tab, x)
   }
-  restore_dplyr_features(df, x)
 }
 collect.Table <- as.data.frame.Table
 collect.RecordBatch <- as.data.frame.RecordBatch
