@@ -53,6 +53,7 @@
 #'
 #' - `$column(i)`: Extract a `ChunkedArray` by integer position from the table
 #' - `$ColumnNames()`: Get all column names (called by `names(tab)`)
+#' - `$RenameColumns(value)`: Set all column names (called by `names(tab) <- value`)
 #' - `$GetColumnByName(name)`: Extract a `ChunkedArray` by string name
 #' - `$field(i)`: Extract a `Field` from the table schema by integer position
 #' - `$SelectColumns(indices)`: Return new `Table` with specified columns, expressed as 0-based integers.
@@ -75,7 +76,7 @@
 #' - `$schema`
 #' - `$metadata`: Returns the key-value metadata of the `Schema` as a named list.
 #'    Modify or replace by assigning in (`tab$metadata <- new_metadata`).
-#'    All list elements are coerced to string.
+#'    All list elements are coerced to string. See `schema()` for more information.
 #' - `$columns`: Returns a list of `ChunkedArray`s
 #' @rdname Table
 #' @name Table
@@ -93,15 +94,19 @@
 Table <- R6Class("Table", inherit = ArrowObject,
   public = list(
     column = function(i) {
-      shared_ptr(ChunkedArray, Table__column(self, i))
+      Table__column(self, i)
     },
     ColumnNames = function() Table__ColumnNames(self),
+    RenameColumns = function(value) Table__RenameColumns(self, value),
     GetColumnByName = function(name) {
       assert_is(name, "character")
       assert_that(length(name) == 1)
-      shared_ptr(ChunkedArray, Table__GetColumnByName(self, name))
+      Table__GetColumnByName(self, name)
     },
-    field = function(i) shared_ptr(Field, Table__field(self, i)),
+    RemoveColumn = function(i) Table__RemoveColumn(self, i),
+    AddColumn = function(i, new_field, value) Table__AddColumn(self, i, new_field, value),
+    SetColumn = function(i, new_field, value) Table__SetColumn(self, i, new_field, value),
+    field = function(i) Table__field(self, i),
 
     serialize = function(output_stream, ...) write_table(self, output_stream, ...),
     ToString = function() ToString_tabular(self),
@@ -110,18 +115,18 @@ Table <- R6Class("Table", inherit = ArrowObject,
       assert_is(target_schema, "Schema")
       assert_is(options, "CastOptions")
       assert_that(identical(self$schema$names, target_schema$names), msg = "incompatible schemas")
-      shared_ptr(Table, Table__cast(self, target_schema, options))
+      Table__cast(self, target_schema, options)
     },
 
     SelectColumns = function(indices) {
-      shared_ptr(Table, Table__SelectColumns(self, indices))
+      Table__SelectColumns(self, indices)
     },
 
     Slice = function(offset, length = NULL) {
       if (is.null(length)) {
-        shared_ptr(Table, Table__Slice1(self, offset))
+        Table__Slice1(self, offset)
       } else {
-        shared_ptr(Table, Table__Slice2(self, offset, length))
+        Table__Slice2(self, offset, length)
       }
     },
     Take = function(i) {
@@ -131,13 +136,13 @@ Table <- R6Class("Table", inherit = ArrowObject,
       if (is.integer(i)) {
         i <- Array$create(i)
       }
-      shared_ptr(Table, call_function("take", self, i))
+      call_function("take", self, i)
     },
     Filter = function(i, keep_na = TRUE) {
       if (is.logical(i)) {
         i <- Array$create(i)
       }
-      shared_ptr(Table, call_function("filter", self, i, options = list(keep_na = keep_na)))
+      call_function("filter", self, i, options = list(keep_na = keep_na))
     },
 
     Equals = function(other, check_metadata = FALSE, ...) {
@@ -150,13 +155,19 @@ Table <- R6Class("Table", inherit = ArrowObject,
 
     ValidateFull = function() {
       Table__ValidateFull(self)
+    },
+
+    invalidate = function() {
+      .Call(`_arrow_Table__Reset`, self)
+      super$invalidate()
     }
+
   ),
 
   active = list(
     num_columns = function() Table__num_columns(self),
     num_rows = function() Table__num_rows(self),
-    schema = function() shared_ptr(Schema, Table__schema(self)),
+    schema = function() Table__schema(self),
     metadata = function(new) {
       if (missing(new)) {
         # Get the metadata (from the schema)
@@ -167,11 +178,11 @@ Table <- R6Class("Table", inherit = ArrowObject,
         out <- Table__ReplaceSchemaMetadata(self, new)
         # ReplaceSchemaMetadata returns a new object but we're modifying in place,
         # so swap in that new C++ object pointer into our R6 object
-        self$set_pointer(out)
+        self$set_pointer(out$pointer())
         self
       }
     },
-    columns = function() map(Table__columns(self), shared_ptr, class = ChunkedArray)
+    columns = function() Table__columns(self)
   )
 )
 
@@ -200,11 +211,24 @@ arrow_attributes <- function(x, only_top_level = FALSE) {
 
   if (is.data.frame(x)) {
     columns <- map(x, arrow_attributes)
-    if (length(att) || !all(map_lgl(columns, is.null))) {
+    out <- if (length(att) || !all(map_lgl(columns, is.null))) {
       list(attributes = att, columns = columns)
     }
-  } else if (length(att)) {
-    list(attributes = att, columns = NULL)
+    return(out)
+  }
+
+  columns <- NULL
+  if (is.list(x) && !inherits(x, "POSIXlt")) {
+    # for list columns, we also keep attributes of each
+    # element in columns
+    columns <- map(x, arrow_attributes)
+    if (all(map_lgl(columns, is.null))) {
+      columns <- NULL
+    }
+  }
+
+  if (length(att) || !is.null(columns)) {
+    list(attributes = att, columns = columns)
   } else {
     NULL
   }
@@ -218,9 +242,9 @@ Table$create <- function(..., schema = NULL) {
   }
   stopifnot(length(dots) > 0)
   if (all_record_batches(dots)) {
-    shared_ptr(Table, Table__from_record_batches(dots, schema))
+    Table__from_record_batches(dots, schema)
   } else {
-    shared_ptr(Table, Table__from_dots(dots, schema))
+    Table__from_dots(dots, schema)
   }
 }
 
@@ -248,12 +272,77 @@ dim.Table <- function(x) c(x$num_rows, x$num_columns)
 #' @export
 names.Table <- function(x) x$ColumnNames()
 
+#' @export
+`names<-.Table` <- function(x, value) x$RenameColumns(value)
+
 #' @export
 `[.Table` <- `[.RecordBatch`
 
 #' @export
 `[[.Table` <- `[[.RecordBatch`
 
+#' @export
+`[[<-.Table` <- function(x, i, value) {
+  if (!is.character(i) & !is.numeric(i)) {
+    stop("'i' must be character or numeric, not ", class(i), call. = FALSE)
+  } else if (is.na(i)) {
+    # Catch if a NA_character or NA_integer is passed. These are caught elsewhere
+    # in cpp (i.e. _arrow_RecordBatch__column_name)
+    # TODO: figure out if catching in cpp like ^^^ is preferred
+    stop("'i' cannot be NA", call. = FALSE)
+  }
+
+  if (is.null(value)) {
+    if (is.character(i)) {
+      i <- match(i, names(x))
+    }
+    x <- x$RemoveColumn(i - 1L)
+  } else {
+    if (!is.character(i)) {
+      # get or create a/the column name
+      if (i <= x$num_columns) {
+        i <- names(x)[i]
+      } else {
+        i <- as.character(i)
+      }
+    }
+
+    # auto-magic recycling on non-ArrowObjects
+    if (!inherits(value, "ArrowObject")) {
+      value <- vctrs::vec_recycle(value, x$num_rows)
+    }
+
+    # construct the field
+    if (!inherits(value, "ChunkedArray")) {
+      value <- chunked_array(value)
+    }
+    new_field <- field(i, value$type)
+
+    if (i %in% names(x)) {
+      i <- match(i, names(x)) - 1L
+      x <- x$SetColumn(i, new_field, value)
+    } else {
+      i <- x$num_columns
+      x <- x$AddColumn(i, new_field, value)
+    }
+  }
+  x
+}
+
+#' @export
+`$<-.Table` <- function(x, i, value) {
+  assert_that(is.string(i))
+  # We need to check if `i` is in names in case it is an active binding (e.g.
+  # `metadata`, in which case we use assign to change the active binding instead
+  # of the column in the table)
+  if (i %in% ls(x)) {
+    assign(i, value, x)
+  } else {
+    x[[i]] <- value
+  }
+  x
+}
+
 #' @export
 `$.Table` <- `$.RecordBatch`
 
