@@ -20,17 +20,99 @@
 #if defined(ARROW_R_WITH_ARROW)
 #include <arrow/type.h>
 
-// [[arrow::export]]
-bool shared_ptr_is_null(SEXP xp) {
-  return reinterpret_cast<std::shared_ptr<void>*>(R_ExternalPtrAddr(xp))->get() ==
-         nullptr;
+namespace cpp11 {
+
+const char* r6_class_name<arrow::DataType>::get(
+    const std::shared_ptr<arrow::DataType>& type) {
+  using arrow::Type;
+
+  switch (type->id()) {
+    case Type::NA:
+      return "Null";
+    case Type::BOOL:
+      return "Boolean";
+    case Type::UINT8:
+      return "UInt8";
+    case Type::UINT16:
+      return "UInt16";
+    case Type::UINT32:
+      return "UInt32";
+    case Type::UINT64:
+      return "UInt64";
+
+    case Type::INT8:
+      return "Int8";
+    case Type::INT16:
+      return "Int16";
+    case Type::INT32:
+      return "Int32";
+    case Type::INT64:
+      return "Int64";
+
+    case Type::HALF_FLOAT:
+      return "Float16";
+    case Type::FLOAT:
+      return "Float32";
+    case Type::DOUBLE:
+      return "Float64";
+
+    case Type::STRING:
+      return "Utf8";
+    case Type::LARGE_STRING:
+      return "LargeUtf8";
+
+    case Type::BINARY:
+      return "Binary";
+    case Type::FIXED_SIZE_BINARY:
+      return "FixedSizeBinary";
+    case Type::LARGE_BINARY:
+      return "LargeBinary";
+
+    case Type::DATE32:
+      return "Date32";
+    case Type::DATE64:
+      return "Date64";
+    case Type::TIMESTAMP:
+      return "Timestamp";
+
+    case Type::TIME32:
+      return "Time32";
+    case Type::TIME64:
+      return "Time64";
+
+    case Type::DECIMAL:
+      return "Decimal128Type";
+
+    case Type::LIST:
+      return "ListType";
+    case Type::LARGE_LIST:
+      return "LargeListType";
+    case Type::FIXED_SIZE_LIST:
+      return "FixedSizeListType";
+
+    case Type::STRUCT:
+      return "StructType";
+    case Type::DICTIONARY:
+      return "DictionaryType";
+
+    default:
+      break;
+  }
+
+  // No R6 classes are defined for:
+  //    INTERVAL
+  //    SPARSE_UNION
+  //    DENSE_UNION
+  //    MAP
+  //    EXTENSION
+  //    DURATION
+  //
+  // If a c++ function returns one it will be wrapped as a DataType.
+
+  return "DataType";
 }
 
-// [[arrow::export]]
-bool unique_ptr_is_null(SEXP xp) {
-  return reinterpret_cast<std::unique_ptr<void>*>(R_ExternalPtrAddr(xp))->get() ==
-         nullptr;
-}
+}  // namespace cpp11
 
 // [[arrow::export]]
 std::shared_ptr<arrow::DataType> Int8__initialize() { return arrow::int8(); }
@@ -126,51 +208,48 @@ std::shared_ptr<arrow::DataType> Time64__initialize(arrow::TimeUnit::type unit)
 }
 
 // [[arrow::export]]
-SEXP list__(SEXP x) {
+std::shared_ptr<arrow::DataType> list__(SEXP x) {
   if (Rf_inherits(x, "Field")) {
     auto field = cpp11::as_cpp<std::shared_ptr<arrow::Field>>(x);
-    return cpp11::as_sexp(arrow::list(field));
+    return arrow::list(field);
   }
 
-  if (Rf_inherits(x, "DataType")) {
-    auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
-    return cpp11::as_sexp(arrow::list(type));
+  if (!Rf_inherits(x, "DataType")) {
+    cpp11::stop("incompatible");
   }
 
-  cpp11::stop("incompatible");
-  return R_NilValue;
+  auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
+  return arrow::list(type);
 }
 
 // [[arrow::export]]
-SEXP large_list__(SEXP x) {
+std::shared_ptr<arrow::DataType> large_list__(SEXP x) {
   if (Rf_inherits(x, "Field")) {
     auto field = cpp11::as_cpp<std::shared_ptr<arrow::Field>>(x);
-    return cpp11::as_sexp(arrow::large_list(field));
+    return arrow::large_list(field);
   }
 
-  if (Rf_inherits(x, "DataType")) {
-    auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
-    return cpp11::as_sexp(arrow::large_list(type));
+  if (!Rf_inherits(x, "DataType")) {
+    cpp11::stop("incompatible");
   }
 
-  cpp11::stop("incompatible");
-  return R_NilValue;
+  auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
+  return arrow::large_list(type);
 }
 
 // [[arrow::export]]
-SEXP fixed_size_list__(SEXP x, int list_size) {
+std::shared_ptr<arrow::DataType> fixed_size_list__(SEXP x, int list_size) {
   if (Rf_inherits(x, "Field")) {
     auto field = cpp11::as_cpp<std::shared_ptr<arrow::Field>>(x);
-    return cpp11::as_sexp(arrow::fixed_size_list(field, list_size));
+    return arrow::fixed_size_list(field, list_size);
   }
 
-  if (Rf_inherits(x, "DataType")) {
-    auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
-    return cpp11::as_sexp(arrow::fixed_size_list(type, list_size));
+  if (!Rf_inherits(x, "DataType")) {
+    cpp11::stop("incompatible");
   }
 
-  cpp11::stop("incompatible");
-  return R_NilValue;
+  auto type = cpp11::as_cpp<std::shared_ptr<arrow::DataType>>(x);
+  return arrow::fixed_size_list(type, list_size);
 }
 
 // [[arrow::export]]
@@ -196,13 +275,12 @@ bool DataType__Equals(const std::shared_ptr<arrow::DataType>& lhs,
 }
 
 // [[arrow::export]]
-int DataType__num_children(const std::shared_ptr<arrow::DataType>& type) {
+int DataType__num_fields(const std::shared_ptr<arrow::DataType>& type) {
   return type->num_fields();
 }
 
 // [[arrow::export]]
-cpp11::writable::list DataType__children_pointer(
-    const std::shared_ptr<arrow::DataType>& type) {
+cpp11::list DataType__fields(const std::shared_ptr<arrow::DataType>& type) {
   return arrow::r::to_r_list(type->fields());
 }
 
