@@ -42,6 +42,22 @@ std::shared_ptr<arrow::Schema> RecordBatch__schema(
   return x->schema();
 }
 
+// [[arrow::export]]
+std::shared_ptr<arrow::RecordBatch> RecordBatch__RenameColumns(
+    const std::shared_ptr<arrow::RecordBatch>& batch,
+    const std::vector<std::string>& names) {
+  int n = batch->num_columns();
+  if (names.size() != static_cast<size_t>(n)) {
+    cpp11::stop("RecordBatch has %d columns but %d names were provided", n, names.size());
+  }
+  std::vector<std::shared_ptr<arrow::Field>> fields(n);
+  for (int i = 0; i < n; i++) {
+    fields[i] = batch->schema()->field(i)->WithName(names[i]);
+  }
+  auto schema = std::make_shared<arrow::Schema>(std::move(fields));
+  return arrow::RecordBatch::Make(schema, batch->num_rows(), batch->columns());
+}
+
 // [[arrow::export]]
 std::shared_ptr<arrow::RecordBatch> RecordBatch__ReplaceSchemaMetadata(
     const std::shared_ptr<arrow::RecordBatch>& x, cpp11::strings metadata) {
@@ -53,14 +69,13 @@ std::shared_ptr<arrow::RecordBatch> RecordBatch__ReplaceSchemaMetadata(
 }
 
 // [[arrow::export]]
-arrow::ArrayVector RecordBatch__columns(
-    const std::shared_ptr<arrow::RecordBatch>& batch) {
+cpp11::list RecordBatch__columns(const std::shared_ptr<arrow::RecordBatch>& batch) {
   auto nc = batch->num_columns();
   arrow::ArrayVector res(nc);
   for (int i = 0; i < nc; i++) {
     res[i] = batch->column(i);
   }
-  return res;
+  return arrow::r::to_r_list(res);
 }
 
 // [[arrow::export]]
