@@ -38,14 +38,13 @@ std::shared_ptr<arrow::RecordBatch> RecordBatchReader__ReadNext(
 // -------- RecordBatchStreamReader
 
 // [[arrow::export]]
-std::shared_ptr<arrow::RecordBatchReader> ipc___RecordBatchStreamReader__Open(
+std::shared_ptr<arrow::ipc::RecordBatchStreamReader> ipc___RecordBatchStreamReader__Open(
     const std::shared_ptr<arrow::io::InputStream>& stream) {
-  std::shared_ptr<arrow::RecordBatchReader> reader;
   return ValueOrStop(arrow::ipc::RecordBatchStreamReader::Open(stream));
 }
 
 // [[arrow::export]]
-std::vector<std::shared_ptr<arrow::RecordBatch>> ipc___RecordBatchStreamReader__batches(
+cpp11::list ipc___RecordBatchStreamReader__batches(
     const std::shared_ptr<arrow::ipc::RecordBatchStreamReader>& reader) {
   std::vector<std::shared_ptr<arrow::RecordBatch>> res;
 
@@ -57,7 +56,7 @@ std::vector<std::shared_ptr<arrow::RecordBatch>> ipc___RecordBatchStreamReader__
     res.push_back(batch);
   }
 
-  return res;
+  return arrow::r::to_r_list(res);
 }
 
 // -------- RecordBatchFileReader
@@ -86,13 +85,22 @@ std::shared_ptr<arrow::RecordBatch> ipc___RecordBatchFileReader__ReadRecordBatch
 // [[arrow::export]]
 std::shared_ptr<arrow::ipc::RecordBatchFileReader> ipc___RecordBatchFileReader__Open(
     const std::shared_ptr<arrow::io::RandomAccessFile>& file) {
-  std::shared_ptr<arrow::ipc::RecordBatchFileReader> reader;
   return ValueOrStop(arrow::ipc::RecordBatchFileReader::Open(file));
 }
 
+// [[arrow::export]]
+std::shared_ptr<arrow::Table> Table__from_RecordBatchReader(
+    const std::shared_ptr<arrow::RecordBatchReader>& reader) {
+  std::shared_ptr<arrow::Table> table = nullptr;
+  StopIfNotOk(reader->ReadAll(&table));
+  return table;
+}
+
 // [[arrow::export]]
 std::shared_ptr<arrow::Table> Table__from_RecordBatchFileReader(
     const std::shared_ptr<arrow::ipc::RecordBatchFileReader>& reader) {
+  // RecordBatchStreamReader inherits from RecordBatchReader
+  // but RecordBatchFileReader apparently does not
   int num_batches = reader->num_record_batches();
   std::vector<std::shared_ptr<arrow::RecordBatch>> batches(num_batches);
   for (int i = 0; i < num_batches; i++) {
@@ -103,21 +111,7 @@ std::shared_ptr<arrow::Table> Table__from_RecordBatchFileReader(
 }
 
 // [[arrow::export]]
-std::shared_ptr<arrow::Table> Table__from_RecordBatchStreamReader(
-    const std::shared_ptr<arrow::ipc::RecordBatchStreamReader>& reader) {
-  std::shared_ptr<arrow::RecordBatch> batch;
-  std::vector<std::shared_ptr<arrow::RecordBatch>> batches;
-  while (true) {
-    StopIfNotOk(reader->ReadNext(&batch));
-    if (!batch) break;
-    batches.push_back(batch);
-  }
-
-  return ValueOrStop(arrow::Table::FromRecordBatches(std::move(batches)));
-}
-
-// [[arrow::export]]
-std::vector<std::shared_ptr<arrow::RecordBatch>> ipc___RecordBatchFileReader__batches(
+cpp11::list ipc___RecordBatchFileReader__batches(
     const std::shared_ptr<arrow::ipc::RecordBatchFileReader>& reader) {
   auto n = reader->num_record_batches();
   std::vector<std::shared_ptr<arrow::RecordBatch>> res(n);
@@ -126,7 +120,7 @@ std::vector<std::shared_ptr<arrow::RecordBatch>> ipc___RecordBatchFileReader__ba
     res[i] = ValueOrStop(reader->ReadRecordBatch(i));
   }
 
-  return res;
+  return arrow::r::to_r_list(res);
 }
 
 #endif
