@@ -24,8 +24,8 @@ test_that("null type works as expected",{
   expect_equal(x$ToString(), "null")
   expect_true(x == x)
   expect_false(x == int8())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
 })
 
 test_that("boolean type work as expected",{
@@ -35,8 +35,8 @@ test_that("boolean type work as expected",{
   expect_equal(x$ToString(), "bool")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 1L)
 })
 
@@ -47,8 +47,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint8")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 8L)
 
   x <- int8()
@@ -57,8 +57,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int8")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 8L)
 
   x <- uint16()
@@ -67,8 +67,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint16")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
   x <- int16()
@@ -77,8 +77,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int16")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
   x <- uint32()
@@ -87,8 +87,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint32")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
   x <- int32()
@@ -97,8 +97,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int32")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
   x <- uint64()
@@ -107,8 +107,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "uint64")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 
   x <- int64()
@@ -117,8 +117,8 @@ test_that("int types works as expected",{
   expect_equal(x$ToString(), "int64")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 })
 
@@ -129,8 +129,8 @@ test_that("float types work as expected",{
   expect_equal(x$ToString(), "halffloat")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 16L)
 
   x <- float32()
@@ -139,8 +139,8 @@ test_that("float types work as expected",{
   expect_equal(x$ToString(), "float")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
 
   x <- float64()
@@ -149,8 +149,8 @@ test_that("float types work as expected",{
   expect_equal(x$ToString(), "double")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
 })
 
@@ -161,8 +161,8 @@ test_that("utf8 type works as expected",{
   expect_equal(x$ToString(), "string")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
 })
 
 test_that("date types work as expected", {
@@ -172,8 +172,8 @@ test_that("date types work as expected", {
   expect_equal(x$ToString(), "date32[day]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$unit(), unclass(DateUnit$DAY))
 
   x <- date64()
@@ -182,8 +182,8 @@ test_that("date types work as expected", {
   expect_equal(x$ToString(), "date64[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$unit(), unclass(DateUnit$MILLI))
 })
 
@@ -194,8 +194,8 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[s]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
   expect_equal(x$unit(), unclass(TimeUnit$SECOND))
@@ -206,8 +206,8 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
   expect_equal(x$unit(), unclass(TimeUnit$MILLI))
@@ -218,8 +218,8 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[us]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
   expect_equal(x$unit(), unclass(TimeUnit$MICRO))
@@ -230,8 +230,8 @@ test_that("timestamp type works as expected", {
   expect_equal(x$ToString(), "timestamp[ns]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$timezone(), "")
   expect_equal(x$unit(), unclass(TimeUnit$NANO))
@@ -248,8 +248,8 @@ test_that("time32 types work as expected", {
   expect_equal(x$ToString(), "time32[s]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
   expect_equal(x$unit(), unclass(TimeUnit$SECOND))
 
@@ -259,8 +259,8 @@ test_that("time32 types work as expected", {
   expect_equal(x$ToString(), "time32[ms]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 32L)
   expect_equal(x$unit(), unclass(TimeUnit$MILLI))
 })
@@ -272,8 +272,8 @@ test_that("time64 types work as expected", {
   expect_equal(x$ToString(), "time64[us]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$unit(), unclass(TimeUnit$MICRO))
 
@@ -283,8 +283,8 @@ test_that("time64 types work as expected", {
   expect_equal(x$ToString(), "time64[ns]")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 0L)
-  expect_equal(x$children(), list())
+  expect_equal(x$num_fields(), 0L)
+  expect_equal(x$fields(), list())
   expect_equal(x$bit_width, 64L)
   expect_equal(x$unit(), unclass(TimeUnit$NANO))
 })
@@ -329,9 +329,9 @@ test_that("list type works as expected", {
   expect_equal(x$ToString(), "list<item: int32>")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 1L)
+  expect_equal(x$num_fields(), 1L)
   expect_equal(
-    x$children(),
+    x$fields(),
     list(field("item", int32()))
   )
   expect_equal(x$value_type, int32())
@@ -345,9 +345,9 @@ test_that("struct type works as expected", {
   expect_equal(x$ToString(), "struct<x: int32, y: bool>")
   expect_true(x == x)
   expect_false(x == null())
-  expect_equal(x$num_children(), 2L)
+  expect_equal(x$num_fields(), 2L)
   expect_equal(
-    x$children(),
+    x$fields(),
     list(field("x", int32()), field("y", boolean()))
   )
   expect_equal(x$GetFieldIndex("x"), 0L)
